@@ -100,7 +100,7 @@
 #include <lal/Units.h>
 #include <lal/Window.h>
 #include <lal/Units.h>
-#include <lal/TFTransform.h>
+#include <lal/EPSearch.h>
 
 
 /*
@@ -1334,7 +1334,7 @@ static GstFlowReturn transform(GstBaseTransform *trans, GstBuffer *inbuf, GstBuf
 		 */
 
 		zero_output_history(element);
-	} else
+	} else if(!gst_audioadapter_is_empty(element->input_queue))
 		g_assert_cmpuint(GST_BUFFER_TIMESTAMP(inbuf), ==, gst_audioadapter_expected_timestamp(element->input_queue));
 	element->next_offset_in = GST_BUFFER_OFFSET_END(inbuf);
 
