@@ -50,7 +50,6 @@ from glue.ligolw import utils
 import lal
 from pylal import datatypes as laltypes
 from pylal import series as lalseries
-from pylal import lalconstants
 
 
 from gstlal import datasource
@@ -61,10 +60,26 @@ from gstlal import simplehandler
 
 ## @file
 # module for reference psds
+#
+# ### Review Status
+#
+# | Names                                       | Hash                                        | Date       |
+# | ------------------------------------------- | ------------------------------------------- | ---------- |
+# | Florent, Sathya, Duncan Me., Jolien, Kipp, Chad | b3ef077fe87b597578000f140e4aa780f3a227aa    | 2014-05-01 |
+#
+#
+# #### Action items
+#
+# - Make graphs of code and compare with gstreamer graphs
+# - Link spectrum movie from DCC
+# - Consider exposing the average samples property
+# - Check FIR kernel normalization norm
+
 
 ## @package python.reference_psd
 # the reference_psd module
 
+
 #
 # =============================================================================
 #
@@ -146,7 +161,7 @@ def measure_psd(gw_data_source_info, instrument, rate, psd_fft_length = 8, verbo
 	head = pipeparts.mkcapsfilter(pipeline, head, "audio/x-raw-float, rate=%d" % rate)
 	head = pipeparts.mkqueue(pipeline, head, max_size_buffers = 8)
 	if gw_data_source_info.seg is not None:
-		average_samples = int(round(float(abs(gw_data_source_info.seg)) / (psd_fft_length / 2) - 1))
+		average_samples = int(round(float(abs(gw_data_source_info.seg)) / (psd_fft_length / 2.) - 1.))
 	else:
 		#FIXME maybe let the user specify this
 		average_samples = 64
@@ -259,7 +274,7 @@ def horizon_distance(psd, m1, m2, snr, f_min, f_max = None, inspiral_spectrum =
 	# clip to ISCO.  see (4) in arXiv:1003.2481
 	#
 
-	f_isco = lalconstants.LAL_C_SI**3 / (6**(3. / 2.) * math.pi * lalconstants.LAL_G_SI * (m1 + m2) * lalconstants.LAL_MSUN_SI)
+	f_isco = lal.C_SI**3 / (6**(3. / 2.) * math.pi * lal.G_SI * (m1 + m2) * lal.MSUN_SI)
 	f_max = min(f_max, f_isco)
 	assert psd.f0 <= f_isco
 	assert psd.f0 <= f_min <= f_isco
@@ -283,7 +298,7 @@ def horizon_distance(psd, m1, m2, snr, f_min, f_max = None, inspiral_spectrum =
 	mu = (m1 * m2) / (m1 + m2)
 	mchirp = mu**(3. / 5.) * (m1 + m2)**(2. / 5.)
 
-	inspiral = (5 * math.pi / (24 * lalconstants.LAL_C_SI**3)) * (lalconstants.LAL_G_SI * mchirp * lalconstants.LAL_MSUN_SI)**(5. / 3.) * (math.pi * f)**(-7. / 3.)
+	inspiral = (5 * math.pi / (24 * lal.C_SI**3)) * (lal.G_SI * mchirp * lal.MSUN_SI)**(5. / 3.) * (math.pi * f)**(-7. / 3.)
 
 	#
 	# SNR for source at D = 1 m <--> D in m for source w/ SNR = 1.  see
@@ -304,7 +319,7 @@ def horizon_distance(psd, m1, m2, snr, f_min, f_max = None, inspiral_spectrum =
 	# D in Mpc for source with desired SNR
 	#
 
-	return D_at_snr_1 / snr / (1e6 * lalconstants.LAL_PC_SI)
+	return D_at_snr_1 / snr / (1e6 * lal.PC_SI)
 
 
 def effective_distance_factor(inclination, fp, fc):
@@ -611,16 +626,21 @@ def interpolate_psd(psd, deltaF):
 
 
 def movingmedian(psd, window_size):
-	data = numpy.copy(psd.data)
+	"""
+	Assumes that the underlying PSD doesn't have variance, i.e., that there
+	is no median / mean correction factor required
+	"""
+	data = psd.data
+	datacopy = numpy.copy(psd.data)
 	for i in range(window_size, len(data)-window_size):
-		data[i] = numpy.median(data[i-window_size:i+window_size])
+		datacopy[i] = numpy.median(data[i-window_size:i+window_size])
 	return laltypes.REAL8FrequencySeries(
 		name = psd.name,
 		epoch = psd.epoch,
 		f0 = psd.f0,
 		deltaF = psd.deltaF,
 		sampleUnits = psd.sampleUnits,
-		data = data
+		data = datacopy
 	)
 
 
