@@ -21,12 +21,19 @@
 from glue import segments
 from glue.ligolw import ligolw
 from glue.ligolw import lsctables
-from glue.ligolw import utils
+from glue.ligolw import utils as ligolw_utils
 from pylal.datatypes import LIGOTimeGPS
 
 
 ## @file
 # the simulation module
+#
+# Review status
+#
+# | Names                                          | Hash                                        | Date       |
+# | -------------------------------------------    | ------------------------------------------- | ---------- |
+# | Sathya, Duncan Me, Jolien, Kipp, Chad          | 7536db9d496be9a014559f4e273e1e856047bf71    | 2014-05-02 |
+#
 
 
 ## @package python.simulation
@@ -57,13 +64,16 @@ def sim_inspiral_to_segment_list(fname, pad=3, verbose=False):
 
 	# Parse the XML file
 
-	xmldoc=utils.load_filename(fname, contenthandler=ContentHandler, verbose=verbose)
+	xmldoc = ligolw_utils.load_filename(fname, contenthandler=ContentHandler, verbose=verbose)
 
 	# extract the padded geocentric end times into segment lists
 
-	sim_inspiral_table=lsctables.table.get_table(xmldoc, lsctables.SimInspiralTable.tableName)
-	for row in sim_inspiral_table:
+	for row in lsctables.SimInspiralTable.get_table(xmldoc):
 		t = LIGOTimeGPS(row.get_time_geocent())
 		seglist.append(segments.segment(t-pad, t+pad))
 
-	return seglist
+	# help the garbage collector
+
+	xmldoc.unlink()
+
+	return seglist.coalesce()
